--[[ Select the quest reward that is worth the most at vendors
     and automatically accept/turn in quests
  ]]

if not FixR then return end

local M = FixR:NewModule("QuestRewardSelect")

local QRSFrame = nil
local QRSFrameEvents = {};
local SelectMostValuable

local function initFrame()
  QRSFrame = CreateFrame("Button", nil, UIParent)
  QRSFrame:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
  QRSFrame:SetFrameStrata("DIALOG")
  QRSFrame:SetWidth(120)
  QRSFrame:SetHeight(22)
  
  QRSFrame:SetText("Choose for me")
  QRSFrame:SetNormalFontObject("GameFontNormal")
  
  local ntex = QRSFrame:CreateTexture()
  ntex:SetTexture("Interface/Buttons/UI-Panel-Button-Up")
  ntex:SetTexCoord(0, 0.625, 0, 0.6875)
  ntex:SetAllPoints()	
  QRSFrame:SetNormalTexture(ntex)
  
  local htex = QRSFrame:CreateTexture()
  htex:SetTexture("Interface/Buttons/UI-Panel-Button-Highlight")
  htex:SetTexCoord(0, 0.625, 0, 0.6875)
  htex:SetAllPoints()
  QRSFrame:SetHighlightTexture(htex)
  
  local ptex = QRSFrame:CreateTexture()
  ptex:SetTexture("Interface/Buttons/UI-Panel-Button-Down")
  ptex:SetTexCoord(0, 0.625, 0, 0.6875)
  ptex:SetAllPoints()
  QRSFrame:SetPushedTexture(ptex)

  QRSFrame:Hide();
  
  QRSFrame:SetScript("OnEvent", function(self, event, ...)
    QRSFrameEvents[event](self, ...); -- call one of the functions above
  end);
  QRSFrame:SetScript("OnClick", function(self, arg1)
    QRSFrame:Hide()
    SelectMostValuable()
  end);
end

function QRSFrameEvents:QUEST_COMPLETE(...)
  --player completes a quest
  local rewardCount = SelectMostValuable()
  if rewardCount <= 1 and M.O["autoComplete"] then -- only complete quest if there is no choice
    QuestFrameCompleteQuestButton:Click()
    if rewardCount == 1 then
      FixR:Echo("You received an item as reward!")
      local warningColor = {
        ["r"] = 0.66667;
        ["g"] = 0.66667;
        ["b"] = 1;
      }
      RaidNotice_AddMessage(RaidWarningFrame, "You received an item as reward!", warningColor, 10)
      --/run PlaySound("TellMessage", "master")
    end
  end
end

function QRSFrameEvents:QUEST_FINISHED(...)
  --Fired whenever the quest frame changes (Detail to Progress to Reward, etc.) or is closed.
  if QRSFrame ~= nil then
    QRSFrame:Hide()
  end
end

function QRSFrameEvents:GOSSIP_SHOW(...)
  --player opens gossip window
  local numActiveQuestList = select("#", GetGossipActiveQuests())
  local numActiveQuest = 0
  local indexQuestComplete = 0

  local numArgs = FixR:IsCat() and 4 or FixR:IsMop() and 5 or 0
  for i = 1, numActiveQuestList, numArgs do
    numActiveQuest = numActiveQuest + 1
    local title, level, isLowLevel, isComplete, isLegendary = select(i, GetGossipActiveQuests())
    if indexQuestComplete == 0 then
      if isComplete then --only select completed quests
        indexQuestComplete = numActiveQuest
      end
    end
  end
  if indexQuestComplete > 0 and M.O["autoComplete"] then
    SelectGossipActiveQuest(indexQuestComplete)
  else
    local numAvailableQuestList = select("#", GetGossipAvailableQuests())
    local indexAvailableQuest = 0
    local numAvailableQuest = 0
    
    numArgs = FixR:IsCat() and 5 or FixR:IsMop() and 6 or 0
    for i = 1, numAvailableQuestList, numArgs do
      numAvailableQuest = numAvailableQuest + 1
      local title, level, isLowLevel, isDaily, isRepeatable, isLegendary = select(i, GetGossipAvailableQuests())
      if indexAvailableQuest == 0 then
        if M.O["autoAcceptLowLevel"] or isLowLevel == nil then --only accept if quest is not lowlevel (unless option is enabled)
          if M.O["autoAcceptDaily"] or isDaily == nil then --accept quest if it's not a daily (unless option is enabled)
            if M.O["autoAcceptRepeatable"] or isRepeatable == nil then --accept quest if it's not repeatable (unless option is enabled)
              indexAvailableQuest = numAvailableQuest
            end
          end
        end
      end
    end
    if indexAvailableQuest > 0 and M.O["autoAccept"] then
      SelectGossipAvailableQuest(indexAvailableQuest)
    end
  end
end

function QRSFrameEvents:QUEST_DETAIL(...)
  --Fired when the player is given a more detailed view of his quest. (after SelectGossipAvailableQuest())
  if M.O["autoAccept"] and (M.O["autoAcceptDaily"] or QuestIsDaily() == nil) then --accept quest if it's not a daily (unless option is enabled)
    AcceptQuest()
  end
end

function QRSFrameEvents:QUEST_GREETING(...)
  --Fired when talking to an NPC that offers or accepts more than one quest, i.e. has more than one active or available quest. (only some NPCs fire this)
  local numActiveQuests = GetNumActiveQuests()
  local indexQuestComplete = 0
  
  for i = 1, numActiveQuests do
    local title, isComplete = GetActiveTitle(i)
    --local isTrivial = IsActiveQuestTrivial(i)
    --new (5.0.1.15799): IsActiveQuestLegendary(i)
    if indexQuestComplete == 0 then
      if isComplete then --only select completed quests
        indexQuestComplete = i
      end
    end
  end
  if indexQuestComplete > 0 and M.O["autoComplete"] then
    SelectActiveQuest(indexQuestComplete)
  else
    local numAvailableQuests = GetNumAvailableQuests()
    local indexAvailableQuest = 0
    
    for i = 1, numAvailableQuests do
      --local title = GetAvailableTitle(i)
      local isTrivial, isDaily, isRepeatable, isLegendary = GetAvailableQuestInfo(i)
      if indexAvailableQuest == 0 then
        if M.O["autoAcceptLowLevel"] or isTrivial == nil then --only accept if quest is not lowlevel (unless option is enabled)
          if M.O["autoAcceptDaily"] or isDaily == nil then --accept quest if it's not a daily (unless option is enabled)
            if M.O["autoAcceptRepeatable"] or isRepeatable == nil then --accept quest if it's not repeatable (unless option is enabled)
              indexAvailableQuest = i
            end
          end
        end
      end
    end
    if indexAvailableQuest > 0 and M.O["autoAccept"] then
      SelectAvailableQuest(indexAvailableQuest)
    end
  end
end

function QRSFrameEvents:QUEST_PROGRESS(...)
  --Fired when a player is talking to an NPC about the status of a quest and has not yet clicked the complete button [its the continue button, not complete]. (after SelectGossipActiveQuest())
  if IsQuestCompletable() and M.O["autoComplete"] then
    CompleteQuest()
  end
end

function M:Load()
  if QRSFrame == nil then  
    initFrame()
  end
  for k, v in pairs(QRSFrameEvents) do
    QRSFrame:RegisterEvent(k); -- Register all events for which handlers have been defined
  end
end
function M:Unload()
  if QRSFrame then
    for k, v in pairs(QRSFrameEvents) do
      QRSFrame:UnregisterEvent(k); -- Unregister all events for which handlers have been defined
    end
  end
end

function M:SetOption(o)
  o = string.lower(o)
  if o == "autocomplete" then
    local bln = not (M.O["autoComplete"] or false)
    M.O["autoComplete"] = bln
    FixR:Echo("QuestRewardSelect: autoComplete: "..(bln and "|cffaaffaaOn" or "|cffff8888Off"))
  elseif o == "autoaccept" then
    local bln = not (M.O["autoAccept"] or false)
    M.O["autoAccept"] = bln
    FixR:Echo("QuestRewardSelect: autoAccept: "..(bln and "|cffaaffaaOn" or "|cffff8888Off"))
  elseif o == "autoacceptdaily" then
    local bln = not (M.O["autoAcceptDaily"] or false)
    M.O["autoAcceptDaily"] = bln
    FixR:Echo("QuestRewardSelect: autoAcceptDaily: "..(bln and "|cffaaffaaOn" or "|cffff8888Off"))
  elseif o == "autoacceptlowlevel" then
    local bln = not (M.O["autoAcceptLowLevel"] or false)
    M.O["autoAcceptLowLevel"] = bln
    FixR:Echo("QuestRewardSelect: autoAcceptLowLevel: "..(bln and "|cffaaffaaOn" or "|cffff8888Off"))
  elseif o == "autoacceptrepeatable" then
    local bln = not (M.O["autoAcceptRepeatable"] or false)
    M.O["autoAcceptRepeatable"] = bln
    FixR:Echo("QuestRewardSelect: autoAcceptRepeatable: "..(bln and "|cffaaffaaOn" or "|cffff8888Off"))
  end
end
function M:ShowOptions()
  FixR:Echo("  QuestRewardSelect autoComplete: "..(M.O["autoComplete"] and "|cffaaffaaOn" or "|cffff8888Off"))
  FixR:Echo("  QuestRewardSelect autoAccept: "..(M.O["autoAccept"] and "|cffaaffaaOn" or "|cffff8888Off"))
  FixR:Echo("  QuestRewardSelect autoAcceptDaily: "..(M.O["autoAcceptDaily"] and "|cffaaffaaOn" or "|cffff8888Off"))
  FixR:Echo("  QuestRewardSelect autoAcceptLowLevel: "..(M.O["autoAcceptLowLevel"] and "|cffaaffaaOn" or "|cffff8888Off"))
  FixR:Echo("  QuestRewardSelect autoAcceptRepeatable: "..(M.O["autoAcceptRepeatable"] and "|cffaaffaaOn" or "|cffff8888Off"))
end

function SelectMostValuable()
  local n = GetNumQuestChoices() --how many different items are available
  if n > 0 then
    local i, iMax, amountMax, linkCount = 1, 0, 0, 0
    for i = 1, n, 1 do
      local _, _, numItems, _, _ = GetQuestItemInfo("choice", i);
      local link = GetQuestItemLink("choice", i);
      if link then
        linkCount = linkCount + 1
        local _, _, _, _, _, _, _, _, _, _, itemSellPrice = GetItemInfo(link);
        local amount = numItems * itemSellPrice
        if amount > amountMax then
          iMax = i
          amountMax = amount
        end
      end
    end
    if linkCount ~= n then
      FixR:Echo("FixR QuestRewardSelect: didn't get an item-link, choose reward for yourself or click fixR button");
      if QuestFrameCompleteQuestButton then
        QRSFrame:Show()
        QRSFrame:ClearAllPoints()
        QRSFrame:SetPoint("BOTTOMLEFT", QuestFrameCompleteQuestButton, "BOTTOMRIGHT", 2, 0)
      end
    elseif iMax > 0 then
      local QuestRewardItemFrame = _G["QuestInfoItem"..iMax]
      if QuestRewardItemFrame then
        QuestRewardItemFrame:Click(); --select most valuable item
      end
    end
  end
  return n
end